from colav_protobuf_utils.protobuf_generator import (
    gen_controller_feedback,
    CTRLMode,
    CTRLStatus,
)
from colav_protobuf.examples import controller_feedback
import pytest


def test_gen_controller_feedback():
    """pytest assertion tests for generation of protobuf controller feedback"""
    proto_utils_controller_feedback = gen_controller_feedback(
        mission_tag=controller_feedback.mission_tag,
        agent_tag=controller_feedback.agent_tag,
        ctrl_mode=CTRLMode(controller_feedback.ctrl_mode),
        ctrl_status=CTRLStatus(controller_feedback.ctrl_status),
        velocity=controller_feedback.ctrl_cmd.velocity,
        yaw_rate=controller_feedback.ctrl_cmd.yaw_rate,
        timestamp=controller_feedback.timestamp,
    )

    assert (
        proto_utils_controller_feedback.mission_tag == controller_feedback.mission_tag
    )
    assert proto_utils_controller_feedback.agent_tag == controller_feedback.agent_tag
    assert proto_utils_controller_feedback.ctrl_mode == controller_feedback.ctrl_mode
    assert (
        proto_utils_controller_feedback.ctrl_status == controller_feedback.ctrl_status
    )
    assert (
        proto_utils_controller_feedback.ctrl_cmd.velocity
        == controller_feedback.ctrl_cmd.velocity
    )
    assert (
        proto_utils_controller_feedback.ctrl_cmd.yaw_rate
        == controller_feedback.ctrl_cmd.yaw_rate
    )
    assert proto_utils_controller_feedback.timestamp == controller_feedback.timestamp



from colav_protobuf.missionRequest_pb2 import MissionRequest
from colav_protobuf.missionResponse_pb2 import MissionResponse

from colav_protobuf.obstaclesUpdate_pb2 import ObstaclesUpdate
from colav_protobuf.agentUpdate_pb2 import AgentUpdate
from typing import List
from enum import Enum

class CTRLMode(Enum):
    UNKOWN = 0
    CRUISE = 1
    T2LOS = 2
    T2Theta = 3
    FB = 4
    WAYPOINT_REACHED = 5


class CTRLStatus(Enum):
    pass


def gen_protobuf_controller_feedback(mission_tag: str, agent_tag: str, ctrl_mode: str):
    pass


@staticmethod
def _gen_state_common_vars(
    state, pose, velocity: float, yaw_rate: float, acceleration: float
):
    """Returns a common state"""
    state.pose = pose
    state.velocity = velocity
    state.yaw_rate = yaw_rate
    state.acceleration = acceleration

    return state


def gen_state(
    pose: AgentUpdate.Pose.Position,
    velocity: float,
    yaw_rate: float,
    acceleration: float,
):
    """Generate Agent State"""
    state = AgentUpdate.State()
    return _gen_state_common_vars(
        state=state,
        pose=pose,
        velocity=velocity,
        yaw_rate=yaw_rate,
        acceleration=acceleration,
    )


def gen_state(
    pose: ObstaclesUpdate.Pose.Position,
    velocity: float,
    yaw_rate: float,
    acceleration: float,
):
    state = ObstaclesUpdate.State()
    return _gen_state_common_vars(
        state=state,
        pose=pose,
        velocity=velocity,
        yaw_rate=yaw_rate,
        acceleration=acceleration,
    )



from colav_protobuf import ControllerFeedback
from enum import Enum

class CTRLMode(Enum):
    UNKNOWN = 0
    CRUISE = 1
    T2LOS = 2
    T2Theta = 3
    FB = 4
    WAYPOINT_REACHED = 5

class CTRLStatus(Enum):
    UNKOWN_STATUS = 0
    ACTIVE = 1
    INACTIVE = 2
    ERROR = 3

def gen_controller_feedback(
        mission_tag: str,
        agent_tag: str,
        ctrl_mode: CTRLMode,
        ctrl_status: CTRLStatus,
        velocity: float,
        yaw_rate: float,
        timestamp: str
):
    feedback = ControllerFeedback()
    feedback.mission_tag = mission_tag
    feedback.agent_tag = agent_tag
    feedback.ctrl_mode = ControllerFeedback.CTRLMode.Value(ctrl_mode.name)
    feedback.ctrl_status = ControllerFeedback.CTRLStatus.Value(ctrl_status.name)
    feedback.ctrl_cmd.velocity = velocity
    feedback.ctrl_cmd.yaw_rate = yaw_rate
    feedback.timestamp = timestamp
    return feedback



from colav_protobuf_utils.protobuf_generator import gen_mission_request, VesselType
from colav_protobuf_utils.protobuf_generator import (
    gen_controller_feedback,
    CTRLStatus,
    CTRLMode,
)
from colav_protobuf_utils.protobuf_generator import gen_agent_update
from colav_protobuf_utils.protobuf_generator import (
    gen_obstacles_update,
    gen_static_obstacle,
    gen_dynamic_obstacle,
    StaticObstacleTypeEnum,
    DynamicObstacleTypeEnum,
)
from colav_protobuf_utils.protobuf_generator import (
    gen_mission_response,
    MissionResponseTypeEnum,
)
from colav_protobuf_utils.serialization import serialise_protobuf
from colav_protobuf_utils.deserialization import deserialise_protobuf

# from colav_protobuf_utils.deserialization import deserialize_protobuf_message


def test_mission_request_protogen_creator():
    return gen_mission_request(
        tag="MOCK_MISSION",
        timestamp="000002000301",
        vessel_tag="MOCK_AGENTS",
        vessel_type=VesselType.HYDROFOIL,
        vessel_max_acceleration=float(5.0),
        vessel_max_deceleration=float(2.0),
        vessel_max_velocity=float(30),
        vessel_min_velocity=float(15.0),
        vessel_max_yaw_rate=float(0.2),
        vessel_loa=float(2.0),
        vessel_beam=float(0.5),
        vessel_safety_radius=float(2.0),
        cartesian_init_position=[float(1), float(1), float(2)],
        cartesian_goal_position=[float(1), float(1), float(5)],
        goal_safety_radius=float(15),
    )


def test_mission_response_protogen_creator():
    return gen_mission_response(
        tag="MOCK_MISSION",
        timestamp="000012300",
        response_type=MissionResponseTypeEnum.MISSION_STARTING,
        response_details="Mission accepted",
    )


def test_agent_update_protogen_creator():
    return gen_agent_update(
        mission_tag="MOCK_MISSION",
        agent_tag="MOCK_AGENT",
        cartesian_position=[0, 0, 0],
        quaternium_orientation=[0, 0, 0, 1],
        velocity=15,
        yaw_rate=0.2,
        acceleration=0.2,
        timestamp="000012300",
    )


def test_dynamic_obstacle_protogen_creator():
    return [
        gen_dynamic_obstacle(
            tag="MOCK_DYNAMIC_OBSTACLE",
            type=DynamicObstacleTypeEnum.VESSEL,
            cartesian_position=[10, 3, 2],
            quaternium_orientation=[0, 0, 0, 1],
            velocity=15,
            yaw_rate=0.2,
            loa=2,
            beam=0.5,
            safety_radius=2,
        )
    ]


def test_static_obstacle_protogen_creator():
    return [
        gen_static_obstacle(
            tag="MOCK_STATIC_OBSTACLE",
            type=StaticObstacleTypeEnum.BUOY,
            cartesian_position=[10, 3, 2],
            quaternium_orientation=[0, 0, 0, 1],
            polyshape_points=[(0, 0, 0), (0, 0, 0), (0, 0, 0)],
            inflation_radius=2,
        )
    ]


def test_obstacles_update_protogen_creator():
    dynamic_obstacle_list = test_dynamic_obstacle_protogen_creator()
    static_obstacle_list = test_static_obstacle_protogen_creator()

    return gen_obstacles_update(
        mission_tag="MOCK_MISSION",
        dynamic_obstacles=dynamic_obstacle_list,
        static_obstacles=static_obstacle_list,
        timestamp="000012300",
    )


def test_controller_feedback_protogen_creator():
    return gen_controller_feedback(
        mission_tag="MOCK MISSION",
        agent_tag="mock agent",
        ctrl_mode=CTRLMode.CRUISE,
        ctrl_status=CTRLStatus.ACTIVE,
        velocity=float(15),
        yaw_rate=float(0.2),
        timestamp="0000012304",
    )


def main():
    proto_mission_request = test_mission_request_protogen_creator()
    print(proto_mission_request)

    proto_mission_response = test_mission_response_protogen_creator()
    print(proto_mission_response)

    proto_agent_update = test_agent_update_protogen_creator()
    print(proto_agent_update)

    proto_obstacles_update = test_obstacles_update_protogen_creator()
    print(proto_obstacles_update)
    serialised_obstacles_update = serialise_protobuf(proto_obstacles_update)
    deserialised_obstacles_update = deserialise_protobuf(serialised_obstacles_update)
    print(deserialised_obstacles_update, type="mission_request")
    proto_controller_feedback = test_controller_feedback_protogen_creator()
    print(proto_controller_feedback)


if __name__ == "__main__":
    main()



import pytest

from colav_protobuf_utils.serialization.serializer import serialise_protobuf

from colav_protobuf.examples import mission_request
from colav_protobuf.examples import mission_response

from colav_protobuf.examples import agent_update
from colav_protobuf.examples import obstacles_update

from colav_protobuf.examples import controller_feedback


@pytest.mark.parametrize(
    "message",
    [
        mission_request,
        mission_response,
        agent_update,
        obstacles_update,
        controller_feedback,
    ],
)
def test_serialiser(message):
    """
    Test serialisation of valid protobuf messages.
    """
    serialized_message = serialise_protobuf(message)
    assert isinstance(serialized_message, bytes), "Serialized message should be bytes"


def test_negative_serialiser():
    """
    Test serialisation with an invalid input.
    """
    with pytest.raises(TypeError):
        serialise_protobuf("test")



from colav_protobuf_utils.protobuf_generator import (
    gen_mission_response,
    MissionResponseTypeEnum,
)
from colav_protobuf.examples import mission_response


def test_proto_gen_mission_response():
    """pytest assertion tests for generation of protobuf mission response"""
    protogen_mission_response = gen_mission_response(
        tag=mission_response.tag,
        timestamp=mission_response.timestamp,
        response_type=MissionResponseTypeEnum(mission_response.response.type),
        response_details=mission_response.response.details,
    )
    assert protogen_mission_response.tag == mission_response.tag
    assert protogen_mission_response.timestamp == mission_response.timestamp
    assert protogen_mission_response.response.type == mission_response.response.type
    assert (
        protogen_mission_response.response.details == mission_response.response.details
    )



from colav_protobuf_utils.protobuf_generator import gen_agent_update
from colav_protobuf.examples import agent_update
import pytest


def get_cartesian_position(state):
    return [state.pose.position.x, state.pose.position.y, state.pose.position.z]


def get_quaternion_orientation(state):
    return [
        state.pose.orientation.x,
        state.pose.orientation.y,
        state.pose.orientation.z,
        state.pose.orientation.w,
    ]


def test_gen_agent_update():
    """pytest assertion tests for generation of protobuf agent update"""
    proto_utils_agent_update = gen_agent_update(
        mission_tag=agent_update.mission_tag,
        agent_tag=agent_update.agent_tag,
        cartesian_position=get_cartesian_position(agent_update.state),
        quaternium_orientation=get_quaternion_orientation(agent_update.state),
        velocity=agent_update.state.velocity,
        yaw_rate=agent_update.state.yaw_rate,
        acceleration=agent_update.state.acceleration,
        timestamp=agent_update.timestamp,
    )

    assert proto_utils_agent_update.mission_tag == agent_update.mission_tag
    assert proto_utils_agent_update.agent_tag == agent_update.agent_tag
    assert (
        proto_utils_agent_update.state.pose.position == agent_update.state.pose.position
    )
    assert (
        proto_utils_agent_update.state.pose.orientation
        == agent_update.state.pose.orientation
    )
    assert proto_utils_agent_update.state.velocity == agent_update.state.velocity
    assert proto_utils_agent_update.state.yaw_rate == agent_update.state.yaw_rate
    assert (
        proto_utils_agent_update.state.acceleration == agent_update.state.acceleration
    )
    assert proto_utils_agent_update.timestamp == agent_update.timestamp



from colav_protobuf import ObstaclesUpdate
from typing import List, Tuple
from enum import Enum


class DynamicObstacleTypeEnum(Enum):
    UNSPECIFIED = ObstaclesUpdate.DynamicObstacleType.DYNAMIC_UNSPECIFIED
    VESSEL = ObstaclesUpdate.DynamicObstacleType.VESSEL


class StaticObstacleTypeEnum(Enum):
    STATIC_UNSPECIFIED = ObstaclesUpdate.StaticObstacleType.STATIC_UNSPECIFIED
    BUOY = ObstaclesUpdate.StaticObstacleType.BUOY
    LAND_MASS = ObstaclesUpdate.StaticObstacleType.LAND_MASS


def gen_dynamic_obstacle(
    tag: str,
    type: DynamicObstacleTypeEnum,
    cartesian_position: Tuple[float, float, float],
    quaternium_orientation: Tuple[float, float, float, float],
    loa: float,
    beam: float,
    safety_radius: float,
    velocity: float,
    yaw_rate: float,
) -> ObstaclesUpdate.DynamicObstacle:
    try:
        dynamic_obstacle = ObstaclesUpdate.DynamicObstacle()
        dynamic_obstacle.tag = tag
        dynamic_obstacle.type = type.value
        dynamic_obstacle.state.pose.position.x = cartesian_position[0]
        dynamic_obstacle.state.pose.position.y = cartesian_position[1]
        dynamic_obstacle.state.pose.position.z = cartesian_position[2]

        dynamic_obstacle.state.pose.orientation.x = quaternium_orientation[0]
        dynamic_obstacle.state.pose.orientation.y = quaternium_orientation[1]
        dynamic_obstacle.state.pose.orientation.z = quaternium_orientation[2]
        dynamic_obstacle.state.pose.orientation.w = quaternium_orientation[3]

        dynamic_obstacle.state.velocity = velocity
        dynamic_obstacle.state.yaw_rate = yaw_rate

        dynamic_obstacle.geometry.loa = loa
        dynamic_obstacle.geometry.beam = beam
        dynamic_obstacle.geometry.safety_radius = safety_radius
    except Exception as e:
        raise e

    return dynamic_obstacle


def gen_static_obstacle(
    tag: str,
    type: StaticObstacleTypeEnum,
    cartesian_position: Tuple[float, float, float],
    quaternium_orientation: Tuple[float, float, float, float],
    polyshape_points: List[Tuple[float, float, float]],
    inflation_radius: float,
) -> ObstaclesUpdate.StaticObstacle:
    try:
        static_obstacle = ObstaclesUpdate.StaticObstacle()
        static_obstacle.tag = tag
        static_obstacle.type = type.value
        static_obstacle.pose.position.x = cartesian_position[0]
        static_obstacle.pose.position.y = cartesian_position[1]
        static_obstacle.pose.position.z = cartesian_position[2]

        static_obstacle.pose.orientation.x = quaternium_orientation[0]
        static_obstacle.pose.orientation.y = quaternium_orientation[1]
        static_obstacle.pose.orientation.z = quaternium_orientation[2]
        static_obstacle.pose.orientation.w = quaternium_orientation[3]

        static_obstacle.geometry.inflation_radius = inflation_radius
        for point in polyshape_points:
            static_obstacle.geometry.polyshape_points.add()
            static_obstacle.geometry.polyshape_points[-1].x = point[0]
            static_obstacle.geometry.polyshape_points[-1].y = point[1]
            static_obstacle.geometry.polyshape_points[-1].z = point[2]
    except Exception as e:
        raise e

    return static_obstacle


def gen_obstacles_update(
    mission_tag: str,
    dynamic_obstacles: List[ObstaclesUpdate.DynamicObstacle],
    static_obstacles: List[ObstaclesUpdate.StaticObstacle],
    timestamp: str,
) -> ObstaclesUpdate:
    """Generates a protobuf message for obstacles update"""
    try:
        obstacles_update = ObstaclesUpdate()
        obstacles_update.mission_tag = mission_tag
        # need to do the repeated thing

        for idx, obstacle in enumerate(dynamic_obstacles):
            obstacles_update.dynamic_obstacles.add()
            obstacles_update.dynamic_obstacles[idx].tag = obstacle.tag
            obstacles_update.dynamic_obstacles[idx].type = obstacle.type
            obstacles_update.dynamic_obstacles[idx].state.pose.position.x = (
                obstacle.state.pose.position.x
            )
            obstacles_update.dynamic_obstacles[idx].state.pose.position.y = (
                obstacle.state.pose.position.y
            )
            obstacles_update.dynamic_obstacles[idx].state.pose.position.z = (
                obstacle.state.pose.position.z
            )
            obstacles_update.dynamic_obstacles[idx].state.pose.orientation.x = (
                obstacle.state.pose.orientation.x
            )
            obstacles_update.dynamic_obstacles[idx].state.pose.orientation.y = (
                obstacle.state.pose.orientation.y
            )
            obstacles_update.dynamic_obstacles[idx].state.pose.orientation.z = (
                obstacle.state.pose.orientation.z
            )
            obstacles_update.dynamic_obstacles[idx].state.pose.orientation.w = (
                obstacle.state.pose.orientation.w
            )
            obstacles_update.dynamic_obstacles[idx].state.velocity = (
                obstacle.state.velocity
            )
            obstacles_update.dynamic_obstacles[idx].state.yaw_rate = (
                obstacle.state.yaw_rate
            )
            obstacles_update.dynamic_obstacles[idx].geometry.loa = obstacle.geometry.loa
            obstacles_update.dynamic_obstacles[idx].geometry.beam = (
                obstacle.geometry.beam
            )
            obstacles_update.dynamic_obstacles[idx].geometry.safety_radius = (
                obstacle.geometry.safety_radius
            )

        for idx, obstacle in enumerate(static_obstacles):
            obstacles_update.static_obstacles.add()
            obstacles_update.static_obstacles[idx].tag = obstacle.tag
            obstacles_update.static_obstacles[idx].type = obstacle.type
            obstacles_update.static_obstacles[idx].pose.position.x = (
                obstacle.pose.position.x
            )
            obstacles_update.static_obstacles[idx].pose.position.y = (
                obstacle.pose.position.y
            )
            obstacles_update.static_obstacles[idx].pose.position.z = (
                obstacle.pose.position.z
            )
            obstacles_update.static_obstacles[idx].pose.orientation.x = (
                obstacle.pose.orientation.x
            )
            obstacles_update.static_obstacles[idx].pose.orientation.y = (
                obstacle.pose.orientation.y
            )
            obstacles_update.static_obstacles[idx].pose.orientation.z = (
                obstacle.pose.orientation.z
            )
            obstacles_update.static_obstacles[idx].pose.orientation.w = (
                obstacle.pose.orientation.w
            )
            obstacles_update.static_obstacles[idx].geometry.inflation_radius = (
                obstacle.geometry.inflation_radius
            )
            for point in obstacle.geometry.polyshape_points:
                obstacles_update.static_obstacles[idx].geometry.polyshape_points.add()
                obstacles_update.static_obstacles[idx].geometry.polyshape_points[
                    -1
                ].x = point.x
                obstacles_update.static_obstacles[idx].geometry.polyshape_points[
                    -1
                ].y = point.y
                obstacles_update.static_obstacles[idx].geometry.polyshape_points[
                    -1
                ].z = point.z

        obstacles_update.timestamp = timestamp
    except Exception as e:
        raise e

    return obstacles_update



import pytest

from colav_protobuf_utils.serialization.serializer import serialise_protobuf
from colav_protobuf_utils.deserialization.deserializer import deserialise_protobuf
from colav_protobuf.examples import mission_request
from colav_protobuf.examples import mission_response

from colav_protobuf.examples import agent_update
from colav_protobuf.examples import obstacles_update

from colav_protobuf.examples import controller_feedback


@pytest.mark.parametrize(
    "message",
    [
        serialise_protobuf(mission_request),
        serialise_protobuf(mission_response),
        serialise_protobuf(agent_update),
        serialise_protobuf(obstacles_update),
        serialise_protobuf(controller_feedback),
    ],
)
def test_deserialiser(message):
    try:
        deserialise_protobuf(message)
        assert True
    except Exception:
        assert False


def test_deserialiser_invalid_message():
    with pytest.raises(Exception):
        deserialise_protobuf(b"invalid message")



from colav_protobuf import AgentUpdate
from enum import Enum
from typing import Tuple

def gen_agent_update(
    mission_tag: str,
    agent_tag: str,
    cartesian_position: Tuple[float, float, float],
    quaternium_orientation: Tuple[float, float, float, float],
    velocity: float,
    yaw_rate: float,
    acceleration: float,
    timestamp: str,
):
    """Generates a protobuf message for agent_update"""
    try:
        update = AgentUpdate()
        update.mission_tag = mission_tag
        update.agent_tag = agent_tag
        update.state.pose.position.x = cartesian_position[0]
        update.state.pose.position.y = cartesian_position[1]
        update.state.pose.position.z = cartesian_position[2]
        update.state.pose.orientation.x = quaternium_orientation[0]
        update.state.pose.orientation.y = quaternium_orientation[1]
        update.state.pose.orientation.z = quaternium_orientation[2]
        update.state.pose.orientation.w = quaternium_orientation[3]
        update.state.velocity = velocity
        update.state.yaw_rate = yaw_rate
        update.state.acceleration = acceleration
        update.timestamp = timestamp
    except Exception as e:
        raise Exception(e)

    return update


from colav_protobuf import MissionResponse
from enum import Enum


class MissionResponseTypeEnum(Enum):
    UNKOWN = MissionResponse.MissionResponseMsg.ResponseTypeEnum.UNKNOWN
    MISSION_STARTING = (
        MissionResponse.MissionResponseMsg.ResponseTypeEnum.MISSION_STARTING
    )
    MISSION_ERROR = MissionResponse.MissionResponseMsg.ResponseTypeEnum.MISSION_ERROR
    MISSION_INVALID = (
        MissionResponse.MissionResponseMsg.ResponseTypeEnum.MISSION_INVALID
    )


def gen_mission_response(
    tag: str,
    timestamp: str,
    response_type: MissionResponseTypeEnum,
    response_details: str,
) -> MissionResponse:
    """Generate a protobuf message for MissionResponse"""
    mission_response = MissionResponse()
    mission_response.tag = tag
    mission_response.timestamp = timestamp
    mission_response.response.type = response_type.value
    mission_response.response.details = response_details

    return mission_response



from functools import singledispatch
from colav_protobuf.missionRequest_pb2 import MissionRequest
from colav_protobuf.missionResponse_pb2 import MissionResponse
from colav_protobuf.obstaclesUpdate_pb2 import ObstaclesUpdate
from colav_protobuf.agentUpdate_pb2 import AgentUpdate
from colav_protobuf.controllerFeedback_pb2 import ControllerFeedback
from enum import Enum


class ProtoType(Enum):
    MISSION_REQUEST = "mission_request"
    MISSION_RESPONSE = "mission_response"
    AGENT_UPDATE = "agent_update"
    OBSTACLES_UPDATE = "obstacles_update"
    CONTROLLER_FEEDBACK = "controller_feedback"


@singledispatch
def deserialise_protobuf(protobuf):
    """Generic deserialization function for colav protobuf messages"""
    raise TypeError(f"Unsupported protobuf type: {type(protobuf)}")


@deserialise_protobuf.register
def _(protobuf: bytes) -> MissionRequest:
    msg = MissionRequest()
    msg.ParseFromString(protobuf)
    return msg


@deserialise_protobuf.register
def _(protobuf: bytes) -> MissionResponse:
    msg = MissionResponse()
    msg.ParseFromString(protobuf)
    return msg


@deserialise_protobuf.register
def _(protobuf: bytes) -> AgentUpdate:
    msg = AgentUpdate()
    msg.ParseFromString(protobuf)
    return msg


@deserialise_protobuf.register
def _(protobuf: bytes):
    msg = ObstaclesUpdate()
    msg.ParseFromString(protobuf)
    return msg


@deserialise_protobuf.register
def _(protobuf: bytes):
    msg = ControllerFeedback()
    msg.ParseFromString(protobuf)
    return msg



from .mission_request import gen_mission_request, VesselType
from .mission_response import gen_mission_response, MissionResponseTypeEnum

from .agent_update import gen_agent_update
from .obstacles_update import (
    gen_obstacles_update,
    gen_dynamic_obstacle,
    gen_static_obstacle,
    DynamicObstacleTypeEnum,
    StaticObstacleTypeEnum,
)

from .controller_feedback import gen_controller_feedback, CTRLMode, CTRLStatus

__all__ = [
    "gen_mission_request",
    "VesselType",
    "gen_mission_response",
    "MissionResponseTypeEnum",
    "gen_agent_update",
    "gen_obstacles_update",
    "gen_dynamic_obstacle",
    "DynamicObstacleTypeEnum",
    "gen_static_obstacle",
    "StaticObstacleTypeEnum",
    "gen_controller_feedback",
    "CTRLMode",
    "CTRLStatus",
]



from colav_protobuf import MissionRequest
from typing import Tuple
from enum import Enum


class VesselType(Enum):
    UNSPECIFIED = 0
    HYDROFOIL = 1


def gen_mission_request(
    tag: str,
    timestamp: str,
    vessel_tag: str,
    vessel_type: VesselType,
    vessel_max_acceleration: float,
    vessel_max_deceleration: float,
    vessel_max_velocity: float,
    vessel_min_velocity: float,
    vessel_max_yaw_rate: float,
    vessel_loa: float,
    vessel_beam: float,
    vessel_safety_radius: float,
    cartesian_init_position: Tuple[float, float, float],
    cartesian_goal_position: Tuple[float, float, float],
    goal_safety_radius: float,
) -> MissionRequest:
    """Generates a protobuf message for MissionRequest"""
    req = MissionRequest()
    try:
        req.tag = tag
        req.timestamp = timestamp
        req.vessel.tag = vessel_tag
        req.vessel.type = MissionRequest.Vessel.VesselType.Value(vessel_type.name)
        req.vessel.constraints.max_acceleration = vessel_max_acceleration
        req.vessel.constraints.max_deceleration = vessel_max_deceleration
        req.vessel.constraints.max_velocity = vessel_max_velocity
        req.vessel.constraints.min_velocity = vessel_min_velocity
        req.vessel.constraints.max_yaw_rate = vessel_max_yaw_rate
        req.vessel.geometry.loa = vessel_loa
        req.vessel.geometry.beam = vessel_beam
        req.vessel.geometry.safety_radius = vessel_safety_radius
        req.init_position.x = cartesian_init_position[0]
        req.init_position.y = cartesian_init_position[1]
        req.init_position.z = cartesian_init_position[2]
        req.goal_waypoint.position.x = cartesian_goal_position[0]
        req.goal_waypoint.position.y = cartesian_goal_position[1]
        req.goal_waypoint.position.z = cartesian_goal_position[2]
        req.goal_waypoint.safety_radius = goal_safety_radius  # TODO: Change this to be acceptance radius instead of safety radius.
    except Exception as e:
        raise Exception(e)

    return req



from functools import singledispatch
from colav_protobuf import MissionRequest
from colav_protobuf import MissionResponse
from colav_protobuf import ObstaclesUpdate
from colav_protobuf import AgentUpdate
from colav_protobuf import ControllerFeedback


@singledispatch
def serialise_protobuf(protobuf) -> bytes:
    """Generic Serialization function for colav protobuf messages."""
    raise TypeError(f"Unsupported protobuf type: {type(protobuf)}")


@serialise_protobuf.register
def _(protobuf: MissionRequest) -> bytes:
    try:
        return protobuf.SerializeToString()
    except Exception as e:
        raise Exception(f"Error serializing MissionRequest: {e}")


@serialise_protobuf.register
def _(protobuf: MissionResponse) -> bytes:
    try:
        return protobuf.SerializeToString()
    except Exception as e:
        raise Exception(f"Error serializing MissionResponse: {e}")


@serialise_protobuf.register
def _(protobuf: AgentUpdate) -> bytes:
    try:
        return protobuf.SerializeToString()
    except Exception as e:
        raise Exception(f"Error serializing AgentUpdate: {e}")


@serialise_protobuf.register
def _(protobuf: ObstaclesUpdate) -> bytes:
    try:
        return protobuf.SerializeToString()
    except Exception as e:
        raise Exception(f"Error serializing ObstaclesUpdate: {e}")


@serialise_protobuf.register
def _(protobuf: ControllerFeedback) -> bytes:
    try:
        return protobuf.SerializeToString()
    except Exception as e:
        raise Exception(f"Error serializing ControllerFeedback: {e}")



from enum import Enum


class VesselType(Enum):
    UNSPECIFIED = 0
    HYDROFOIL = 1
    OTHER = 2


def gen_vessel(
    tag: str,
    type: VesselType,
    max_acceleration: float,
    max_deceleration: float,
    max_velocity: float,
    min_velocity: float,
    max_yaw_rate: float,
    loa: float,  # Length Overall
    beam: float,  # breadth
    safety_radius: float,
):
    pass



from colav_protobuf_utils.protobuf_generator import gen_mission_request, VesselType
from colav_protobuf.examples import mission_request
import pytest


def test_gen_mission_request():
    proto_utils_mission_request = gen_mission_request(
        tag=mission_request.tag,
        timestamp=mission_request.timestamp,
        vessel_tag=mission_request.vessel.tag,
        vessel_type=VesselType(mission_request.vessel.type),
        vessel_max_acceleration=mission_request.vessel.constraints.max_acceleration,
        vessel_max_deceleration=mission_request.vessel.constraints.max_deceleration,
        vessel_max_velocity=mission_request.vessel.constraints.max_velocity,
        vessel_min_velocity=mission_request.vessel.constraints.min_velocity,
        vessel_max_yaw_rate=mission_request.vessel.constraints.max_yaw_rate,
        vessel_loa=mission_request.vessel.geometry.loa,
        vessel_beam=mission_request.vessel.geometry.beam,
        vessel_safety_radius=mission_request.vessel.geometry.safety_radius,
        cartesian_init_position=(
            mission_request.init_position.x,
            mission_request.init_position.y,
            mission_request.init_position.z,
        ),
        cartesian_goal_position=(
            mission_request.goal_waypoint.position.x,
            mission_request.goal_waypoint.position.y,
            mission_request.goal_waypoint.position.z,
        ),
        goal_safety_radius=mission_request.goal_waypoint.safety_radius,
    )
    assert proto_utils_mission_request.tag == mission_request.tag
    assert proto_utils_mission_request.timestamp == mission_request.timestamp
